import { Interface } from "ethers";
import { EVMFunctionParameter, EVMFunctionSignature, } from "../../businessObjects/EVMFunctionSignature.js";
import { ChainTransaction } from "../../businessObjects/versioned/ChainTransaction.js";
import { SuiAccountAddress, EVMAccountAddressRegex, } from "../../primitives/index.js";
/**
 * This is a concrete implementation of the Transaction class from Ethers. I'd really prefer to not have to
 * import and depend on Ethers directly, but odds are any project that is using @snickerdoodlelabs/objects
 * is also importing Ethers.
 * Docs are here: https://docs.ethers.io/v5/api/utils/transactions/#Transaction
 */
export class SuiTransaction extends ChainTransaction {
    constructor(chain, hash, timestamp, to, from, value, gasPrice, contractAddress, input, methodId, functionName, events, measurementDate) {
        super(chain, hash, timestamp, measurementDate);
        this.chain = chain;
        this.hash = hash;
        this.timestamp = timestamp;
        this.to = to;
        this.from = from;
        this.value = value;
        this.gasPrice = gasPrice;
        this.contractAddress = contractAddress;
        this.input = input;
        this.methodId = methodId;
        this.functionName = functionName;
        this.measurementDate = measurementDate;
        this.functionSignature = null;
        let addrs = new Set();
        if (this.to) {
            addrs.add(this.to);
        }
        if (this.from) {
            addrs.add(this.from);
        }
        if (this.input &&
            this.functionName &&
            this.methodId &&
            !this.functionName.startsWith(this.methodId)) {
            try {
                const iface = new Interface([`function ${this.functionName}`]);
                const func = iface.getFunction(this.input.slice(0, 10));
                if (func == null) {
                    throw new Error(`While constructing SuiTransaction, function not found on interface ${this.functionName} for input ${this.input}`);
                }
                const paramValues = iface.decodeFunctionData(func.name, this.input);
                // filter out unrecognized methodIDs
                // we may want to do a secondary lookup on failure here:
                // https://www.4byte.directory/docs/
                this.functionSignature = new EVMFunctionSignature(func.name, func.type, func.inputs.map((input, i) => {
                    return new EVMFunctionParameter(input.name, input.type, paramValues[i]);
                }));
                addrs = new Set([
                    ...addrs,
                    ...this._getDescendants(this.functionSignature),
                ]);
            }
            catch (e) {
                // this is also a bit noisy
                console.warn("error decoding transaction input", e);
            }
        }
        if (events) {
            addrs = new Set([...addrs, ...this._getDescendants(events)]);
        }
        this.accountAddresses = Array.from(addrs);
    }
    _getDescendants(obj) {
        let result = new Set();
        for (const [key, value] of Object.entries(obj)) {
            if (value && typeof value === "object") {
                result = new Set([...result, ...this._getDescendants(value)]);
            }
            else {
                if (typeof value === "string" && value.match(EVMAccountAddressRegex)) {
                    result.add(SuiAccountAddress(value));
                }
            }
        }
        return result;
    }
}
//# sourceMappingURL=SuiTransaction.js.map